@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemGMPPlugin.cpp,v 1.14 2006/10/24 22:58:50 hadess Exp $
+ * $Id: totemGMPPlugin.cpp 3790 2006-12-15 23:08:57Z chpe $
  */
 
 #include <mozilla-config.h>
@@ -62,7 +62,9 @@ static const totemPluginMimeEntry kMimeTypes[] = {
 	{ "audio/x-ms-asx", "asx", NULL, TRUE },
 	{ "video/x-ms-wma", "wma", NULL, TRUE },
 	{ "audio/x-ms-wax", "wax", NULL, TRUE },
-	{ "video/mpeg", "mpg", NULL, TRUE }
+	{ "video/mpeg", "mpg", NULL, TRUE },
+	{ "audio/mpeg", "mp3", NULL, TRUE },
+	{ "audio/x-mpegurl", "m3u", NULL, TRUE }
 };
 
 totemScriptablePlugin::totemScriptablePlugin (totemPlugin *aPlugin)
@@ -208,7 +210,7 @@ totemScriptablePlugin::Pause ()
 {
   NS_ENSURE_STATE (IsValid ());
 
-  return mPlugin->Pause ();
+  return mPlugin->DoCommand (TOTEM_COMMAND_PAUSE);
 }
 
 NS_IMETHODIMP
@@ -216,7 +218,7 @@ totemScriptablePlugin::Play ()
 {
   NS_ENSURE_STATE (IsValid ());
 
-  return mPlugin->Play ();
+  return mPlugin->DoCommand (TOTEM_COMMAND_PLAY);
 }
 
 NS_IMETHODIMP
@@ -224,5 +226,5 @@ totemScriptablePlugin::Stop ()
 {
   NS_ENSURE_STATE (IsValid ());
 
-  return mPlugin->Stop ();
+  return mPlugin->DoCommand (TOTEM_COMMAND_STOP);
 }
