@@ -191,6 +191,7 @@ totem_gtk_plug_get_toplevel (GtkPlug *plug)
 
 	do
 	{
+		/* FIXME: multi-head */
 		if (XQueryTree (GDK_DISPLAY (), xid, &root,
 					&parent, &children, &nchildren) == 0)
 		{
@@ -220,7 +221,7 @@ totem_interface_set_transient_for (GtkWindow *window, GtkWindow *parent)
 		if (toplevel != NULL) {
 			gdk_window_set_transient_for
 				(GTK_WIDGET (window)->window, toplevel);
-			gdk_window_unref (toplevel);
+			g_object_unref (toplevel);
 		}
 	} else {
 		gtk_window_set_transient_for (GTK_WINDOW (window),
@@ -228,3 +229,28 @@ totem_interface_set_transient_for (GtkWindow *window, GtkWindow *parent)
 	}
 }
 
+char *
+totem_interface_get_license (void)
+{
+	const char const *license[] = {
+		N_("Totem is free software; you can redistribute it and/or modify "
+		   "it under the terms of the GNU General Public License as published by "
+		   "the Free Software Foundation; either version 2 of the License, or "
+		   "(at your option) any later version."),
+		N_("Totem is distributed in the hope that it will be useful, "
+		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+		   "GNU General Public License for more details."),
+		N_("You should have received a copy of the GNU General Public License "
+		   "along with Totem; if not, write to the Free Software Foundation, Inc., "
+		   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
+		N_("Totem contains an exception to allow the use of proprietary "
+		   "GStreamer plugins.")
+	};
+	return g_strconcat (_(license[0]), "\n\n",
+			    _(license[1]), "\n\n",
+			    _(license[2]), "\n\n",
+			    _(license[3]), "\n",
+			    NULL);
+}
+
