@@ -475,6 +475,54 @@ totem_compare_recent_items (GtkRecentInfo *a, GtkRecentInfo *b)
 	return 0;
 }
 
+/* Copied from eel so we don't have a dependency on them.
+ * It's being consolidated into glib eventually anyway.
+ */
+static char *
+totem_str_middle_truncate (const char *string,
+			      guint truncate_length)
+{
+	char *truncated;
+	guint length;
+	guint num_left_chars;
+	guint num_right_chars;
+
+	const char delimter[] = "...";
+	const guint delimter_length = strlen (delimter);
+	const guint min_truncate_length = delimter_length + 2;
+
+	if (string == NULL) {
+		return NULL;
+	}
+
+	/* It doesnt make sense to truncate strings to less than
+	 * the size of the delimiter plus 2 characters (one on each
+	 * side)
+	 */
+	if (truncate_length < min_truncate_length) {
+		return g_strdup (string);
+	}
+
+	length = strlen (string);
+
+	/* Make sure the string is not already small enough. */
+	if (length <= truncate_length) {
+		return g_strdup (string);
+	}
+
+	/* Find the 'middle' where the truncation will occur. */
+	num_left_chars = (truncate_length - delimter_length) / 2;
+	num_right_chars = truncate_length - num_left_chars - delimter_length + 1;
+
+	truncated = g_new (char, truncate_length + 1);
+
+	strncpy (truncated, string, num_left_chars);
+	strncpy (truncated + num_left_chars, delimter, delimter_length);
+	strncpy (truncated + num_left_chars + delimter_length, string + length - num_right_chars + 1, num_right_chars);
+	
+	return truncated;
+}
+
 static void
 totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *totem)
 {
@@ -493,15 +541,16 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 	totem->recent_action_group = gtk_action_group_new ("recent-action-group");
 	gtk_ui_manager_insert_action_group (totem->ui_manager,
 			totem->recent_action_group, -1);
- 
+
 	items = gtk_recent_manager_get_items (recent_manager);
 	items = g_list_sort (items, (GCompareFunc) totem_compare_recent_items);
 
 	for (l = items; l && l->data; l = g_list_next (l)) {
 		GtkRecentInfo *info;
 		GtkAction     *action;
-		gchar         *action_name;
-		gchar         *label;
+		char          *action_name;
+		char          *label;
+		const char    *uri;
 
 		info = (GtkRecentInfo *) l->data;
 
@@ -509,12 +558,21 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 			continue;
 
 		action_name = g_strdup_printf ("recent-file%u", i++);
-		label = g_strdup_printf ("_%d.  %s", n_items + 1,
+		uri = gtk_recent_info_get_uri (info);
+		if (strstr (uri, "file:///") == NULL)
+			label = g_strdup_printf ("_%d.  %s", n_items + 1, uri);
+		else {
+			label = g_strdup_printf ("_%d.  %s", n_items + 1,
 				gtk_recent_info_get_display_name (info));
+		}
+
+		/* Truncate the menu item if it's too long */
+		char *label_trunc = totem_str_middle_truncate (label, TOTEM_MAX_RECENT_ITEM_LEN);
+		g_free (label);
 
 		action = g_object_new (GTK_TYPE_ACTION,
 			       "name", action_name,
-			       "label", label,
+			       "label", label_trunc,
 			       NULL);
 
 		g_object_set_data_full (G_OBJECT (action), "recent-info",
@@ -530,11 +588,11 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 
 		gtk_ui_manager_add_ui (totem->ui_manager, totem->recent_ui_id,
 			       "/tmw-menubar/movie/recent-placeholder",
-			       label, action_name, GTK_UI_MANAGER_MENUITEM,
+			       label_trunc, action_name, GTK_UI_MANAGER_MENUITEM,
 			       FALSE);
 
 		g_free (action_name);
-		g_free (label);
+		g_free (label_trunc);
 
 		if (++n_items == 5)
 			break;
@@ -564,26 +622,39 @@ void
 totem_action_add_recent (Totem *totem, const char *filename)
 {
 	GtkRecentData data;
-	char *groups[] = { "Totem", NULL };
-	char *display;
+	char *groups[] = { NULL, NULL };
 
-	if (strstr (filename, "file:///") == NULL)
-		return;
-
-	display = g_filename_from_uri (filename, NULL, NULL);
-	if (display) {
-		data.display_name = g_filename_display_basename (display);
-		g_free (display);
-	}
-	data.description = NULL;
 	data.mime_type = gnome_vfs_get_mime_type (filename);
+
+	if (strstr (filename, "file:///") == NULL) {
+		/* It's a URI/stream */
+		groups[0] = "TotemStreams";
+		data.display_name = NULL;
+	} else {
+		char *display;
+
+		/* Local files with no mime-type probably don't exist */
+		if (data.mime_type == NULL)
+			return;
+
+		/* It's a local file */
+		display = g_filename_from_uri (filename, NULL, NULL);
+		if (display) {
+			data.display_name = g_filename_display_basename (display);
+			g_free (display);
+		}
+		groups[0] = "Totem";
+	}
+
+	data.description = NULL;
 	data.app_name = g_strdup (g_get_application_name ());
 	data.app_exec = g_strjoin (" ", g_get_prgname (), "%u", NULL);
 	data.groups = groups;
 	gtk_recent_manager_add_full (totem->recent_manager,
 			filename, &data);
 
-	g_free (data.display_name);
+	if (data.display_name != NULL)
+		g_free (data.display_name);
 	g_free (data.mime_type);
 	g_free (data.app_name);
 	g_free (data.app_exec);
@@ -630,10 +701,23 @@ my_gnome_vfs_volume_get_mount_path (GnomeVFSVolume *volume)
 	return path;
 }
 
+static char *
+escape_device_name_for_menu (const char *name)
+{
+	char *new, **a;
+
+	a = g_strsplit (name, "_", -1);
+	new = g_strjoinv ("__", a);
+	g_strfreev (a);
+
+	return new;
+}
+
 static void
 add_device_to_menu (GObject *device, guint position, Totem *totem)
 {
-	char *name, *icon_name, *device_path, *label, *activation_uri;
+	char *name, *escaped_name, *icon_name, *device_path;
+	char *label, *activation_uri;
 	GtkAction *action;
 	gboolean disabled = FALSE;
 
@@ -646,6 +730,13 @@ add_device_to_menu (GObject *device, guint position, Totem *totem)
 			disabled = TRUE;
 		}
 		g_free (activation_uri);
+	} else {
+		if (GNOME_IS_VFS_DRIVE (device)) {
+			device_path = gnome_vfs_drive_get_device_path
+				(GNOME_VFS_DRIVE (device));
+			disabled = !totem_cd_has_medium (device_path);
+			g_free (device_path);
+		}
 	}
 
 	name = fake_gnome_vfs_device_get_something (device,
@@ -657,9 +748,11 @@ add_device_to_menu (GObject *device, guint position, Totem *totem)
 		&my_gnome_vfs_volume_get_mount_path,
 		&gnome_vfs_drive_get_device_path);
 
-	g_strchomp (name);
-	label = g_strdup_printf (_("Play Disc '%s'"), name);
+	g_strstrip (name);
+	escaped_name = escape_device_name_for_menu (name);
 	g_free (name);
+	label = g_strdup_printf (_("Play Disc '%s'"), escaped_name);
+	g_free (escaped_name);
 
 	name = g_strdup_printf (_("device%d"), position);
 	action = gtk_action_new (name, label, NULL, NULL);
@@ -800,6 +893,12 @@ eject_action_callback (GtkAction *action, Totem *totem)
 	totem_action_eject (totem);
 }
 
+static void
+properties_action_callback (GtkAction *action, Totem *totem)
+{
+	totem_action_show_properties (totem);
+}
+
 static void
 play_action_callback (GtkAction *action, Totem *totem)
 {
@@ -968,6 +1067,7 @@ about_action_callback (GtkAction *action, Totem *totem)
 		"Chee Bin Hoh <cbhoh@gnome.org>",
 		NULL
 	};
+	char *license = totem_interface_get_license ();
 
 #ifdef HAVE_GTK_ONLY
 	frontend_type = N_("GTK+");
@@ -990,10 +1090,13 @@ about_action_callback (GtkAction *action, Totem *totem)
 				     "artists", artists,
 				     "translator-credits", _("translator-credits"),
 				     "logo-icon-name", "totem",
+				     "license", license,
+				     "wrap-license", TRUE,
 				     NULL);
 
 	g_free (backend_version);
 	g_free (description);
+	g_free (license);
 }
 
 static void
@@ -1062,16 +1165,25 @@ aspect_ratio_changed_callback (GtkRadioAction *action, GtkRadioAction *current,
 	totem_action_set_aspect_ratio (totem, gtk_radio_action_get_current_value (current));
 }
 
+static void
+clear_playlist_action_callback (GtkAction *action, Totem *totem)
+{
+	totem_playlist_clear (totem->playlist);
+	totem_action_set_mrl (totem, NULL);
+}
+
 static const GtkActionEntry entries[] = {
 	{ "movie-menu", NULL, N_("_Movie") },
 	{ "open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O", N_("Open a file"), G_CALLBACK (open_action_callback) },
 	{ "open-location", NULL, N_("Open _Location..."), "<control>L", N_("Open a non-local file"), G_CALLBACK (open_location_action_callback) },
 	{ "eject", "media-eject", N_("_Eject"), "<control>E", NULL, G_CALLBACK (eject_action_callback) },
+	{ "properties", GTK_STOCK_PROPERTIES, N_("_Properties"), "<control>P", NULL, G_CALLBACK (properties_action_callback) },
 	{ "play", GTK_STOCK_MEDIA_PLAY, N_("Play / Pa_use"), "P", N_("Play or pause the movie"), G_CALLBACK (play_action_callback) },
 	{ "quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Quit the program"), G_CALLBACK (quit_action_callback) },
 
 	{ "edit-menu", NULL, N_("_Edit") },
 	{ "take-screenshot", "camera-photo", N_("Take _Screenshot..."), "<control>S", N_("Take a screenshot"), G_CALLBACK (take_screenshot_action_callback) },
+	{ "clear-playlist", NULL, N_("Clear Playlist"), NULL, N_("Clear Playlist"), G_CALLBACK (clear_playlist_action_callback) },
 	{ "preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, NULL, G_CALLBACK (preferences_action_callback) },
 
 	{ "view-menu", NULL, N_("_View") },
@@ -1205,6 +1317,13 @@ totem_ui_manager_setup (Totem *totem)
 	gtk_action_group_add_action (totem->main_action_group, action);
 	g_object_unref (action);
 
+	/* Hide help if we're using GTK+ only */
+#ifdef HAVE_GTK_ONLY
+	action = gtk_action_group_get_action
+		(totem->main_action_group, "contents");
+	gtk_action_set_visible (action, FALSE);
+#endif /* HAVE_GTK_ONLY */
+
 	totem->zoom_action_group = gtk_action_group_new ("zoom-action-group");
 	gtk_action_group_set_translation_domain (totem->zoom_action_group,
 			GETTEXT_PACKAGE);
