@@ -67,9 +67,6 @@ struct TotemPlaylistPrivate
 	GtkTreeSelection *selection;
 	TotemPlParser *parser;
 
-	/* This is the playing icon */
-	GdkPixbuf *icon;
-
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
 
@@ -80,14 +77,7 @@ struct TotemPlaylistPrivate
 	char *path;
 	char *save_path;
 
-	/* Repeat mode */
-	gboolean repeat;
-
-	/* Reorder Flag */
-	gboolean drag_started;
-
 	/* Shuffle mode */
-	gboolean shuffle;
 	int *shuffled;
 	int current_shuffled, shuffle_len;
 
@@ -95,7 +85,19 @@ struct TotemPlaylistPrivate
 
 	int x, y;
 
-	gboolean disable_save_to_disk;
+	guint disable_save_to_disk : 1;
+
+	/* Repeat mode */
+	guint repeat : 1;
+
+	/* Reorder Flag */
+	guint drag_started : 1;
+
+	/* Drop disabled flag */
+	guint drop_disabled : 1;
+
+	/* Shuffle mode */
+	guint shuffle : 1;
 };
 
 /* Signals */
@@ -110,7 +112,7 @@ enum {
 };
 
 enum {
-	PIX_COL,
+	PLAYING_COL,
 	FILENAME_COL,
 	URI_COL,
 	TITLE_CUSTOM_COL,
@@ -142,8 +144,6 @@ static const GtkActionEntry entries[] = {
 	{ "copy-location", GTK_STOCK_COPY, N_("_Copy location"), NULL, N_("Copy the location to the clipboard"), G_CALLBACK (playlist_copy_location_action_activated) }
 };
 
-static GtkWidgetClass *parent_class = NULL;
-
 static void totem_playlist_class_init (TotemPlaylistClass *class);
 static void totem_playlist_init       (TotemPlaylist      *playlist);
 
@@ -575,8 +575,8 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 	{
 		GtkTreeIter iter;
 		char *index;
-		GdkPixbuf *pixbuf;
 		GtkTreePath *path;
+		gboolean playing;
 
 		index = g_strdup_printf ("%d", i);
 		if (gtk_tree_model_get_iter_from_string
@@ -588,13 +588,10 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 		}
 		g_free (index);
 
-		gtk_tree_model_get (playlist->_priv->model, &iter,
-				PIX_COL, &pixbuf, -1);
-
-		if (pixbuf == NULL)
+		gtk_tree_model_get (playlist->_priv->model, &iter, PLAYING_COL, &playing, -1);
+		if (!playing) {
 			continue;
-
-		gdk_pixbuf_unref (pixbuf);
+		}
 
 		/* Only emit the changed signal if we changed the ->current */
 		path = gtk_tree_path_new_from_indices (i, -1);
@@ -616,10 +613,16 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 static gboolean 
 button_press_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 { 
+	TotemPlaylist *playlist = (TotemPlaylist *)data;
+
+	if (playlist->_priv->drop_disabled)
+		return FALSE;
+
+	playlist->_priv->drop_disabled = TRUE;
 	gtk_drag_dest_unset (treeview);
 	g_signal_handlers_block_by_func (treeview, (GFunc) drop_cb, data);
 
-	totem_playlist_set_reorderable ((TotemPlaylist *) data, TRUE);
+	totem_playlist_set_reorderable (playlist, TRUE);
 
 	return FALSE;
 }
@@ -629,8 +632,9 @@ button_release_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
-	if (!playlist->_priv->drag_started)
+	if (!playlist->_priv->drag_started && playlist->_priv->drop_disabled)
 	{
+		playlist->_priv->drop_disabled = FALSE;
 		totem_playlist_set_reorderable (playlist, FALSE);
 		gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 				target_table, G_N_ELEMENTS (target_table),
@@ -648,7 +652,7 @@ drag_begin_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
-	playlist->_priv->drag_started= TRUE;
+	playlist->_priv->drag_started = TRUE;
 
 	return;
 }
@@ -658,6 +662,7 @@ drag_end_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
+	playlist->_priv->drop_disabled = FALSE;
 	playlist->_priv->drag_started = FALSE;
 	totem_playlist_set_reorderable (playlist, FALSE);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, target_table,
@@ -725,18 +730,12 @@ static void
 totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
 	GSList *filenames, *l;
-	char *new_path;
 
 	filenames = totem_add_files (totem_playlist_get_toplevel (playlist),
-			playlist->_priv->path, &new_path);
+			NULL);
 	if (filenames == NULL)
 		return;
 
-	if (new_path != NULL) {
-		g_free (playlist->_priv->path);
-		playlist->_priv->path = new_path;
-	}
-
 	for (l = filenames; l != NULL; l = l->next) {
 		char *mrl;
 
@@ -893,7 +892,7 @@ totem_playlist_save_playlist (TotemPlaylist *playlist, char *filename, gint acti
 							save_types[active_format - 1].type);
 	else {
 		for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-			if(g_str_has_suffix (filename, save_types[i].suffix)) {
+			if (g_str_has_suffix (filename, save_types[i].suffix)) {
 				cur = &save_types[i];
 				break;
 			}
@@ -916,10 +915,12 @@ totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
 	gtk_widget_show (label);
 
 	combo_box = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _("By extension"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
+			_("By extension"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
 	for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), save_types[i].name);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
+				save_types[i].name);
 	}
 	gtk_widget_show (combo_box);
 
@@ -928,6 +929,13 @@ totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
 	gtk_widget_show (hbox);
 	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc), hbox);
 
+	atk_object_add_relationship (gtk_widget_get_accessible (label),
+			ATK_RELATION_LABEL_FOR,
+			gtk_widget_get_accessible (combo_box));
+	atk_object_add_relationship (gtk_widget_get_accessible (combo_box),
+			ATK_RELATION_LABELLED_BY,
+			gtk_widget_get_accessible (label));
+
 	return combo_box;
 }
 
@@ -1150,7 +1158,19 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 }
 
 static void
-init_columns (GtkTreeView *treeview)
+set_playing_icon (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
+		GtkTreeModel *model, GtkTreeIter *iter, TotemPlaylist *playlist)
+{
+	gboolean playing;
+
+	gtk_tree_model_get (model, iter, PLAYING_COL, &playing, -1);
+
+	g_object_set (renderer, "icon-name",
+		      playing ? "audio-volume-medium" : NULL, NULL);
+}
+
+static void
+init_columns (GtkTreeView *treeview, TotemPlaylist *playlist)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -1159,8 +1179,9 @@ init_columns (GtkTreeView *treeview)
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
-	gtk_tree_view_column_set_attributes (column, renderer,
-			"pixbuf", PIX_COL, NULL);
+	gtk_tree_view_column_set_cell_data_func (column, renderer,
+			(GtkTreeCellDataFunc) set_playing_icon, playlist, NULL);
+	g_object_set (renderer, "stock-size", GTK_ICON_SIZE_MENU, NULL);
 	gtk_tree_view_append_column (treeview, column);
 
 	/* Labels */
@@ -1211,8 +1232,16 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 			totem_playlist_table_signals[CHANGED], 0,
 			NULL);
 
-	g_signal_handlers_unblock_by_func (treeview,
-			(GFunc) drop_cb, playlist);
+	if (playlist->_priv->drop_disabled) {
+		playlist->_priv->drop_disabled = FALSE;
+		totem_playlist_set_reorderable (playlist, FALSE);
+		gtk_drag_dest_set (GTK_WIDGET (treeview), GTK_DEST_DEFAULT_ALL,
+				target_table, G_N_ELEMENTS (target_table),
+				GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+		g_signal_handlers_unblock_by_func (treeview,
+				(GFunc) drop_cb, playlist);
+	}
 }
 
 static void
@@ -1223,7 +1252,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 
 	/* the model */
 	model = GTK_TREE_MODEL (gtk_list_store_new (NUM_COLS,
-				GDK_TYPE_PIXBUF,
+				G_TYPE_BOOLEAN,
 				G_TYPE_STRING,
 				G_TYPE_STRING,
 				G_TYPE_BOOLEAN,
@@ -1237,7 +1266,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
 	g_object_unref (G_OBJECT (model));
 
-	init_columns (GTK_TREE_VIEW (treeview));
+	init_columns (GTK_TREE_VIEW (treeview), playlist);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
@@ -1259,7 +1288,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 			G_CALLBACK (button_release_cb), playlist);
 	g_signal_connect (G_OBJECT (treeview), "drag_begin",
                         G_CALLBACK (drag_begin_cb), playlist);
-         g_signal_connect (G_OBJECT (treeview), "drag_end",
+	g_signal_connect (G_OBJECT (treeview), "drag_end",
                         G_CALLBACK (drag_end_cb), playlist);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 			target_table, G_N_ELEMENTS (target_table),
@@ -1277,7 +1306,7 @@ update_repeat_cb (GConfClient *client, guint cnxn_id,
 	gboolean repeat;
 
 	repeat = gconf_value_get_bool (entry->value);
-	playlist->_priv->repeat = repeat;
+	playlist->_priv->repeat = (repeat != FALSE);
 
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
@@ -1384,7 +1413,7 @@ update_lockdown (GConfClient *client, guint cnxn_id,
 {
 	playlist->_priv->disable_save_to_disk = gconf_client_get_bool
 			(playlist->_priv->gc,
-			"/desktop/gnome/lockdown/disable_save_to_disk", NULL);
+			"/desktop/gnome/lockdown/disable_save_to_disk", NULL) != FALSE;
 	totem_playlist_update_save_button (playlist);
 }
 
@@ -1395,7 +1424,7 @@ init_config (TotemPlaylist *playlist)
 
 	playlist->_priv->disable_save_to_disk = gconf_client_get_bool
 	       		(playlist->_priv->gc,
-			"/desktop/gnome/lockdown/disable_save_to_disk", NULL);
+			"/desktop/gnome/lockdown/disable_save_to_disk", NULL) != FALSE;
 	totem_playlist_update_save_button (playlist);
 
 	gconf_client_add_dir (playlist->_priv->gc, GCONF_PREFIX,
@@ -1415,9 +1444,9 @@ init_config (TotemPlaylist *playlist)
 			playlist, NULL, NULL);
 
 	playlist->_priv->repeat = gconf_client_get_bool (playlist->_priv->gc,
-			GCONF_PREFIX"/repeat", NULL);
+			GCONF_PREFIX"/repeat", NULL) != FALSE;
 	playlist->_priv->shuffle = gconf_client_get_bool (playlist->_priv->gc,
-			GCONF_PREFIX"/shuffle", NULL);
+			GCONF_PREFIX"/shuffle", NULL) != FALSE;
 }
 
 static void
@@ -1450,12 +1479,8 @@ totem_playlist_finalize (GObject *object)
 {
 	TotemPlaylist *playlist = TOTEM_PLAYLIST (object);
 
-	g_return_if_fail (object != NULL);
-
 	if (playlist->_priv->current != NULL)
 		gtk_tree_path_free (playlist->_priv->current);
-	if (playlist->_priv->icon != NULL)
-		gdk_pixbuf_unref (playlist->_priv->icon);
 	g_object_unref (playlist->_priv->parser);
 
 	if (playlist->_priv->ui_manager != NULL) {
@@ -1466,9 +1491,7 @@ totem_playlist_finalize (GObject *object)
 		g_object_unref (G_OBJECT (playlist->_priv->action_group));
 	}
 
-	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
-		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
-	}
+	G_OBJECT_CLASS (totem_playlist_parent_class)->finalize (object);
 }
 
 GtkWidget*
@@ -1478,7 +1501,7 @@ totem_playlist_new (void)
 	GtkWidget *container, *item;
 	char *filename;
 
-	playlist = TOTEM_PLAYLIST (g_object_new (GTK_TYPE_PLAYLIST, NULL));
+	playlist = TOTEM_PLAYLIST (g_object_new (TOTEM_TYPE_PLAYLIST, NULL));
 
 	playlist->_priv->xml = totem_interface_load ("playlist.glade",
 			_("playlist"), TRUE, NULL);
@@ -1555,9 +1578,6 @@ totem_playlist_new (void)
 	/* The configuration */
 	init_config (playlist);
 
-	playlist->_priv->icon =
-		totem_interface_load_pixbuf ("playlist-playing.png");
-
 	gtk_widget_show_all (GTK_WIDGET (playlist));
 
 	return GTK_WIDGET (playlist);
@@ -1571,7 +1591,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	GtkTreeIter iter;
 	char *filename_for_display, *uri;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
 	if (display_name == NULL)
@@ -1588,7 +1608,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 
 	store = GTK_LIST_STORE (playlist->_priv->model);
 	gtk_list_store_insert_with_values (store, &iter, G_MAXINT32,
-			PIX_COL, NULL,
+			PLAYING_COL, FALSE,
 			FILENAME_COL, filename_for_display,
 			URI_COL, uri,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
@@ -1638,12 +1658,15 @@ totem_playlist_add_mrl (TotemPlaylist *playlist, const char *mrl,
 	return TRUE;
 }
 
-void
+gboolean
 totem_playlist_clear (TotemPlaylist *playlist)
 {
 	GtkListStore *store;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
+
+	if (PL_LEN == 0)
+		return FALSE;
 
 	store = GTK_LIST_STORE (playlist->_priv->model);
 	gtk_list_store_clear (store);
@@ -1651,6 +1674,10 @@ totem_playlist_clear (TotemPlaylist *playlist)
 	if (playlist->_priv->current != NULL)
 		gtk_tree_path_free (playlist->_priv->current);
 	playlist->_priv->current = NULL;
+
+	totem_playlist_update_save_button (playlist);
+
+	return TRUE;
 }
 
 static void
@@ -1788,7 +1815,7 @@ char
 	GtkTreeIter iter;
 	char *path;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), NULL);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return NULL;
@@ -1811,7 +1838,7 @@ char
 	GtkTreeIter iter;
 	char *path;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), NULL);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return NULL;
@@ -1837,7 +1864,7 @@ totem_playlist_get_current_metadata (TotemPlaylist *playlist,
 {
 	GtkTreeIter iter;
 
-        g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1862,7 +1889,7 @@ totem_playlist_has_previous_mrl (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1891,7 +1918,7 @@ totem_playlist_has_next_mrl (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1921,7 +1948,7 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean f
 	GtkTreeIter iter;
 	gboolean custom_title;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1960,7 +1987,7 @@ totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 	GtkTreeIter iter;
 	GtkTreePath *path;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1972,14 +1999,9 @@ totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 
 	g_return_val_if_fail (&iter != NULL, FALSE);
 
-	if (state != FALSE)
-		gtk_list_store_set (store, &iter,
-				PIX_COL, playlist->_priv->icon,
-				-1);
-	else
-		gtk_list_store_set (store, &iter,
-				PIX_COL, NULL,
-				-1);
+	gtk_list_store_set (store, &iter,
+			PLAYING_COL, state,
+			-1);
 
 	if (state == FALSE)
 		return TRUE;
@@ -1998,7 +2020,7 @@ totem_playlist_set_previous (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	if (totem_playlist_has_previous_mrl (playlist) == FALSE)
 		return;
@@ -2048,7 +2070,7 @@ totem_playlist_set_next (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	if (totem_playlist_has_next_mrl (playlist) == FALSE)
 	{
@@ -2084,7 +2106,7 @@ totem_playlist_set_next (TotemPlaylist *playlist)
 gboolean
 totem_playlist_get_repeat (TotemPlaylist *playlist)
 {
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	return playlist->_priv->repeat;
 }
@@ -2092,7 +2114,7 @@ totem_playlist_get_repeat (TotemPlaylist *playlist)
 void
 totem_playlist_set_repeat (TotemPlaylist *playlist, gboolean repeat)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/repeat",
 			repeat, NULL);
@@ -2101,7 +2123,7 @@ totem_playlist_set_repeat (TotemPlaylist *playlist, gboolean repeat)
 gboolean
 totem_playlist_get_shuffle (TotemPlaylist *playlist)
 {
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	return playlist->_priv->shuffle;
 }
@@ -2109,7 +2131,7 @@ totem_playlist_get_shuffle (TotemPlaylist *playlist)
 void
 totem_playlist_set_shuffle (TotemPlaylist *playlist, gboolean shuffle)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/shuffle",
 			shuffle, NULL);
@@ -2118,7 +2140,7 @@ totem_playlist_set_shuffle (TotemPlaylist *playlist, gboolean shuffle)
 void
 totem_playlist_set_at_start (TotemPlaylist *playlist)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	totem_playlist_unset_playing (playlist);
 
@@ -2135,7 +2157,7 @@ totem_playlist_set_at_end (TotemPlaylist *playlist)
 {
 	int indice;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	totem_playlist_unset_playing (playlist);
 
@@ -2163,7 +2185,7 @@ totem_playlist_get_current (TotemPlaylist *playlist)
 	char *path;
 	double index;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), 0);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), 0);
 
 	path = gtk_tree_path_to_string (playlist->_priv->current);
 	if (path == NULL)
@@ -2178,7 +2200,7 @@ totem_playlist_get_current (TotemPlaylist *playlist)
 guint
 totem_playlist_get_last (TotemPlaylist *playlist)
 {
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), -1);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), -1);
 
 	return PL_LEN - 1;
 }
@@ -2186,7 +2208,7 @@ totem_playlist_get_last (TotemPlaylist *playlist)
 void
 totem_playlist_set_current (TotemPlaylist *playlist, guint index)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	if (index >= (guint) PL_LEN)
 		return;
@@ -2200,9 +2222,9 @@ totem_playlist_set_current (TotemPlaylist *playlist, guint index)
 static void
 totem_playlist_class_init (TotemPlaylistClass *klass)
 {
-	parent_class = gtk_type_class (gtk_dialog_get_type ());
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	G_OBJECT_CLASS (klass)->finalize = totem_playlist_finalize;
+	object_class->finalize = totem_playlist_finalize;
 
 	/* Signals */
 	totem_playlist_table_signals[CHANGED] =
