@@ -20,7 +20,7 @@
  * permission are above and beyond the permissions granted by the GPL license
  * Totem is covered by.
  *
- * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * Monday 7th February 2005: Christian Schaller: Add exception clause.
  * See license_change file for details.
  *
  */
@@ -30,8 +30,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <popt.h>
-
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
@@ -48,7 +46,10 @@
 #define BORING_IMAGE_VARIANCE 256.0		/* Tweak this if necessary */
 
 gboolean finished = FALSE;
-static char *format = "png";
+static gboolean jpeg_output = FALSE;
+static gboolean output_size = 128;
+static gboolean time_limit = TRUE;
+static char **filenames = NULL;
 
 #ifdef THUMB_DEBUG
 static void
@@ -80,13 +81,13 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 
 	if (holes == NULL)
 	{
-		gdk_pixbuf_ref (pixbuf);
+		g_object_ref (pixbuf);
 		return pixbuf;
 	}
 
 	g_assert (gdk_pixbuf_get_has_alpha (pixbuf) == FALSE);
 	g_assert (gdk_pixbuf_get_has_alpha (holes) != FALSE);
-	target = gdk_pixbuf_ref (pixbuf);
+	target = g_object_ref (pixbuf);
 
 	for (i = 0; i < height; i += gdk_pixbuf_get_height (holes))
 	{
@@ -97,7 +98,7 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 	}
 
 	tmp = gdk_pixbuf_flip (holes, FALSE);
-	gdk_pixbuf_unref (holes);
+	g_object_unref (holes);
 	holes = tmp;
 
 	for (i = 0; i < height; i += gdk_pixbuf_get_height (holes))
@@ -110,7 +111,7 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 				1, 1, GDK_INTERP_NEAREST, 255);
 	}
 
-	gdk_pixbuf_unref (holes);
+	g_object_unref (holes);
 
 	return target;
 }
@@ -132,7 +133,7 @@ add_holes_to_pixbuf_large (GdkPixbuf *pixbuf, int size)
 
 	if (left == NULL)
 	{
-		gdk_pixbuf_ref (pixbuf);
+		g_object_ref (pixbuf);
 		return pixbuf;
 	}
 
@@ -143,8 +144,8 @@ add_holes_to_pixbuf_large (GdkPixbuf *pixbuf, int size)
 
 	if (right == NULL)
 	{
-		gdk_pixbuf_unref (left);
-		gdk_pixbuf_ref (pixbuf);
+		g_object_unref (left);
+		g_object_ref (pixbuf);
 		return pixbuf;
 	}
 
@@ -279,7 +280,7 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 			with_holes = add_holes_to_pixbuf_small (small,
 					d_width, d_height);
 			g_return_if_fail (with_holes != NULL);
-			gdk_pixbuf_unref (small);
+			g_object_unref (small);
 		} else {
 			with_holes = small;
 		}
@@ -291,7 +292,8 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 	a_width = g_strdup_printf ("%d", width);
 	a_height = g_strdup_printf ("%d", height);
 
-	if (gdk_pixbuf_save (with_holes, path, format, &err,
+	if (gdk_pixbuf_save (with_holes, path,
+				jpeg_output ? "jpeg" : "png", &err,
 				"tEXt::Thumb::Image::Width", a_width,
 				"tEXt::Thumb::Image::Height", a_height,
 				NULL) == FALSE)
@@ -307,7 +309,7 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 			g_print ("totem-video-thumbnailer couln't write the thumbnail '%s' for video '%s'\n", path, video_path);
 		}
 
-		gdk_pixbuf_unref (with_holes);
+		g_object_unref (with_holes);
 		return;
 	}
 
@@ -315,7 +317,7 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 	show_pixbuf (with_holes);
 #endif
 
-	gdk_pixbuf_unref (with_holes);
+	g_object_unref (with_holes);
 }
 
 static gpointer
@@ -332,44 +334,37 @@ time_monitor (gpointer data)
 	exit (0);
 }
 
-static void
-usage (const char *cmd)
-{
-	g_print ("Usage: %s [-s <size>] [-j] <input> <output> [backend options]\n", cmd);
-}
-
-static gboolean
-video_is_still_image (BaconVideoWidget *bvw)
-{
-	GValue value = { 0, };
-
-	bacon_video_widget_get_metadata (bvw, BVW_INFO_STILL_IMAGE, &value);
-	return g_value_get_boolean (&value);
-}
+static const GOptionEntry entries[] = {
+	{ "jpeg", 'j',  0, G_OPTION_ARG_NONE, &jpeg_output, "Output the thumbnail as a JPEG instead of PNG", NULL },
+	{ "size", 's', 0, G_OPTION_ARG_INT, &output_size, "Size of the thumbnail in pixels", NULL },
+	{ "no-limit", 'l', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &time_limit, "Don't limit the thumbnailing time to 30 seconds", NULL },
+	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames, "Movies to index", NULL },
+	{ NULL }
+};
 
 int main (int argc, char *argv[])
 {
+	GOptionGroup *options;
+	GOptionContext *context;
 	GError *err = NULL;
 	BaconVideoWidget *bvw;
 	GdkPixbuf *pixbuf;
-	int i, length, size;
+	int length;
 	char *input, *output;
 
-	const float frame_locations[] =
-	{ 1.0 / 3.0, 2.0 / 3.0, 0.1,0.9, 0.5 };
+	const float frame_locations[] = { 1.0 / 3.0, 2.0 / 3.0, 0.1,0.9, 0.5 };
 	const int frame_locations_length = 5;
 	int current;
 
-	if (argc <= 2 || strcmp (argv[1], "-h") == 0 ||
-	    strcmp (argv[1], "--help") == 0) {
-		usage (argv[0]);
-		return -1;
-	}
-
 #ifdef G_OS_UNIX
 	nice (20);
 #endif
 
+	context = g_option_context_new ("Thumbnail movies");
+	options = bacon_video_widget_get_option_group ();
+	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, options);
+
 	g_thread_init (NULL);
 #ifndef THUMB_DEBUG
 	g_type_init ();
@@ -381,32 +376,19 @@ int main (int argc, char *argv[])
 	gnome_authentication_manager_init ();
 #endif
 
-	bacon_video_widget_init_backend (&argc, &argv);
 	gnome_vfs_init ();
-
-	size = 128;
-	input = output = NULL;
-	i = 1;
-	while (i < argc) {
-		if (strcmp (argv[i], "-s") == 0) {
-			size = g_strtod (argv[++i], NULL);
-			i++;
-			continue;
-		}
-		if (strcmp (argv[i], "-j") == 0) {
-			format = "jpeg";
-			i++;
-			continue;
-		}
-		input = argv[i];
-		output = argv[++i];
-		break;
+	if (g_option_context_parse (context, &argc, &argv, &err) == FALSE) {
+		g_print ("couldn't parse command-line options: %s\n", err->message);
+		g_error_free (err);
+		return 1;
 	}
 
-	if (output == NULL || input == NULL) {
-		usage (argv[0]);
-		return -1;
+	if (filenames == NULL || g_strv_length (filenames) != 2) {
+		g_print ("Expects an input and an output file\n");
+		return 1;
 	}
+	input = filenames[0];
+	output = filenames[1];
 
 	bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new (-1, -1, BVW_USE_TYPE_CAPTURE, &err));
 	if (err != NULL)
@@ -417,7 +399,8 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	g_thread_create (time_monitor, (gpointer) input, FALSE, NULL);
+	if (time_limit != FALSE)
+		g_thread_create (time_monitor, (gpointer) input, FALSE, NULL);
 
 	if (bacon_video_widget_open (bvw, input, &err) == FALSE)
 	{
@@ -428,26 +411,11 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	if (video_is_still_image (bvw) != FALSE) {
-		pixbuf = gdk_pixbuf_new_from_file (input, &err);
-		if (pixbuf != NULL)
-		{
-			save_pixbuf (pixbuf, output, input, size, TRUE);
-			gdk_pixbuf_unref (pixbuf);
-			exit (0);
-		}
-		g_print ("totem-video-thumbnailer couln't open file '%s'\n"
-				"Reason: %s.\n", input, err->message);
-		g_error_free (err);
-		exit (1);
-	}
-
 	bacon_video_widget_play (bvw, &err);
 	if (err != NULL)
 	{
 		g_print ("totem-video-thumbnailer couln't play file: '%s'\n"
-				"Reason: %s.\n",
-				input, err->message);
+				"Reason: %s.\n", input, err->message);
 		g_error_free (err);
 		exit (1);
 	}
@@ -480,13 +448,15 @@ int main (int argc, char *argv[])
 
 		/* Pull the frame, if it's interesting we bail early */
 		pixbuf = bacon_video_widget_get_current_frame (bvw);
-		if (is_image_interesting (pixbuf) != FALSE)
+		if (pixbuf != NULL && is_image_interesting (pixbuf) != FALSE)
 			break;
 
-		/* If we get to the end of this loop, we'll end up using the last image
-		 * we pulled */
-		if(current + 1 < frame_locations_length)
-			gdk_pixbuf_unref (pixbuf);
+		/* If we get to the end of this loop, we'll end up using
+		 * the last image we pulled */
+		if(current + 1 < frame_locations_length) {
+			if (pixbuf != NULL)
+				g_object_unref (pixbuf);
+		}
 	}
 
 	/* Cleanup */
@@ -501,8 +471,8 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	save_pixbuf (pixbuf, output, input, size, FALSE);
-	gdk_pixbuf_unref (pixbuf);
+	save_pixbuf (pixbuf, output, input, output_size, FALSE);
+	g_object_unref (pixbuf);
 
 	return 0;
 }
